@@ -8,7 +8,7 @@ Please find a description of this package at:
 
 Author: Ilan Schnell
 """
-__version__ = '0.2.4'
+__version__ = '0.2.5'
 
 from _bitarray import _bitarray, bits2bytes
 
@@ -83,7 +83,7 @@ fromfile, tostring, fromstring
     
     def __add__(self, other):
         """
-        Concatenate two bitarray objetcs.
+        Concatenate two bitarray objects.
         The endianness of the result is the endianness of the first bitarray.
         """
         if not isinstance(other, bitarray):
@@ -103,26 +103,14 @@ fromfile, tostring, fromstring
         return self
     
     def __mul__(self, n):
-        if not isinstance(n, (int, long)):
-            raise TypeError('can repeat integer times')
-        res = bitarray(endian=self.endian())
-        for dum in xrange(n):
-            res.extend(self)
+        res = self.copy()
+        res._multiply(n)
         return res
     
     __rmul__ = __mul__
     
     def __imul__(self, n):
-        if not isinstance(n, (int, long)):
-            raise TypeError('can repeat integer times')
-        if n <= 0:
-            self[:] = bitarray(endian=self.endian())
-        elif n == 1:
-            pass
-        else: # n > 1
-            tmp = bitarray(self)
-            for dum in xrange(1, n):
-                self.extend(tmp)
+        self._multiply(n)
         return self
 
     # ----------------------------------------------------------
