@@ -504,7 +504,7 @@ extend_string(bitarrayobject *self, PyObject *string, int kind)
 {
     Py_ssize_t addbits, i;
     char c, buff[256], *str, *item;
-    int vi;
+    int vi = 0;
 
     assert (PyString_Check(string));
     addbits = PyString_Size(string);
@@ -1005,13 +1005,65 @@ Invert all bits in the array (in-place),\n\
 i.e. convert each 1-bit into a 0-bit and vice versa.");
 
 
+static PyObject *
+bitarray_multiply(bitarrayobject *self, PyObject *v)
+{
+    idx_t vi;
+    
+    if (!ISINDEX(v)) {
+	PyErr_SetString(PyExc_TypeError,
+			"integer value expected for multiplication");
+	return NULL;
+    }
+    getIndex(v, &vi);
+    
+    if (vi <= 0) {
+	delete(self, 0, self->nbits);
+	Py_RETURN_NONE;
+    }
+    if (vi == 1)
+	Py_RETURN_NONE;
+    
+    /* vi > 1 */
+    Py_ssize_t newsize;
+    idx_t newbitsize, j;
+    char *item;
+    
+    newbitsize = self->nbits * vi;
+    check_overflow(newbitsize);
+    newsize = BYTES(newbitsize);
+    item = self->ob_item;
+    
+    PyMem_RESIZE(item, char, newsize);
+    if (item == NULL) {
+	PyErr_NoMemory();
+	return NULL;
+    }
+    self->ob_item = item;
+    self->ob_size = newsize;
+    self->allocated = self->ob_size;
+    
+    for (j = 1; j < vi; j++)
+	bitcopy(self, j * self->nbits, self, 0, self->nbits);
+    
+    self->nbits = newbitsize;
+    Py_RETURN_NONE;
+}
+
+PyDoc_STRVAR(multiply_doc,
+"_multiply(value)\n\
+\n\
+Perform in-place multiplication (repetition) of bitarray, value must be\n\
+and integer.  This method is not meant to be called directly, but from\n\
+__mul__ and __imul__");
+
+
 static PyObject *
 bitarray_bitwise(bitarrayobject *self, PyObject *args)
 {
-    PyObject *other;
+    bitarrayobject *other;
     char *operation;
     Py_ssize_t i;
-    bitarrayobject *s, *o;
     
     if (!PyArg_ParseTuple(args, "Os:_bitwise", &other, &operation))
         return NULL;
@@ -1022,29 +1074,27 @@ bitarray_bitwise(bitarrayobject *self, PyObject *args)
 	return NULL;
     }
     
-    s = (bitarrayobject *) self;
-    o = (bitarrayobject *) other;
-    if (s->nbits != o->nbits) {
+    if (self->nbits != other->nbits) {
 	PyErr_SetString(PyExc_ValueError,
 			"bitarrays of equal length expected "
 			"for bitwise operation");
 	return NULL;
     }
     
-    setunused(s);
-    setunused(o);
+    setunused(self);
+    setunused(other);
     
     if (strcmp(operation, "and") == 0) {
 	for (i = 0; i < self->ob_size; i++)
-	    s->ob_item[i] &= o->ob_item[i];
+	    self->ob_item[i] &= other->ob_item[i];
     }
     else if (strcmp(operation, "or") == 0) {
 	for (i = 0; i < self->ob_size; i++)
-	    s->ob_item[i] |= o->ob_item[i];
+	    self->ob_item[i] |= other->ob_item[i];
     }
     else if (strcmp(operation, "xor") == 0) {
 	for (i = 0; i < self->ob_size; i++)
-	    s->ob_item[i] ^= o->ob_item[i];
+	    self->ob_item[i] ^= other->ob_item[i];
     }
     else {
 	PyErr_SetString(PyExc_ValueError, "bitwise operation unknown");
@@ -1059,7 +1109,7 @@ PyDoc_STRVAR(bitwise_doc,
 \n\
 Perform in-place bitwise operations, operations is a string specifying the\n\
 operation ('and', 'or', 'xor').  This method is not meant to be called\n\
-directly, but from, it is called by __and__, __iadd__, etc...");
+directly, but from __and__, __iand__, etc");
 
 
 static PyObject *
@@ -1097,19 +1147,32 @@ Set all bits in the bitarray to bool(x).");
 
 
 static PyObject *
-bitarray_sort(bitarrayobject *self)
+bitarray_sort(bitarrayobject *self, PyObject *args, PyObject *kwds)
 {
-    idx_t n, n0;
+    idx_t n, n0, n1;
+    int reverse = 0;
+    static char* kwlist[] = {"reverse", NULL};
 
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|i:sort", kwlist, &reverse))
+        return NULL;
+    
     n = self->nbits;
-    n0 = n - count(self);
-    setrange(self, 0, n0, 0);
-    setrange(self, n0, n, 1);
+    n1 = count(self);
+
+    if (reverse) {
+	setrange(self, 0, n1, 1);
+	setrange(self, n1, n, 0);
+    }
+    else {
+	n0 = n - n1;	
+	setrange(self, 0, n0, 0);
+	setrange(self, n0, n, 1);
+    }
     Py_RETURN_NONE;
 }
 
 PyDoc_STRVAR(sort_doc,
-"sort()\n\
+"sort(reverse=False)\n\
 \n\
 Sort the bits in the array (in-place).");
 
@@ -1337,7 +1400,7 @@ PyDoc_STRVAR(unpack_doc,
 "unpack(zero='\\x00', one='\\xff')\n\
 \n\
 Return a string containing one character for each bit in the bitarray,\n\
-using the specifield mapping.\n\
+using the specified mapping.\n\
 Note that unpack('0', '1') has the same effect as to01().\n\
 See also the pack method.");
 
@@ -1730,7 +1793,7 @@ btree_traverse(PyObject *iter, PyObject *tree)
    and can therefore not be symbols in the dict keys.
 */
 static PyObject *
-btree_new()
+btree_new(void)
 {
     PyObject *list;
     
@@ -1833,82 +1896,85 @@ decode the content of the bitarray and return the list of symbols.");
 
 static PyMethodDef
 bitarray_methods[] = {
-    {"all",          (PyCFunction)bitarray_all,         METH_NOARGS,
+    {"all",          (PyCFunction) bitarray_all,         METH_NOARGS,
      all_doc},
-    {"any",          (PyCFunction)bitarray_any,         METH_NOARGS,
+    {"any",          (PyCFunction) bitarray_any,         METH_NOARGS,
      any_doc},
-    {"append",       (PyCFunction)bitarray_append,      METH_O,
+    {"append",       (PyCFunction) bitarray_append,      METH_O,
      append_doc},
-    {"_bitwise",     (PyCFunction)bitarray_bitwise,     METH_VARARGS,
+    {"_bitwise",     (PyCFunction) bitarray_bitwise,     METH_VARARGS,
      bitwise_doc},
-    {"buffer_info",  (PyCFunction)bitarray_buffer_info, METH_NOARGS,
+    {"buffer_info",  (PyCFunction) bitarray_buffer_info, METH_NOARGS,
      buffer_info_doc},
-    {"bytereverse",  (PyCFunction)bitarray_bytereverse, METH_NOARGS,
+    {"bytereverse",  (PyCFunction) bitarray_bytereverse, METH_NOARGS,
      bytereverse_doc},
-    {"__contains__", (PyCFunction)bitarray_contains,    METH_O,
+    {"__contains__", (PyCFunction) bitarray_contains,    METH_O,
      contains_doc},
-    {"copy",         (PyCFunction)bitarray_copy,        METH_NOARGS,
+    {"copy",         (PyCFunction) bitarray_copy,        METH_NOARGS,
      copy_doc},
-    {"__copy__",     (PyCFunction)bitarray_copy,        METH_NOARGS,
+    {"__copy__",     (PyCFunction) bitarray_copy,        METH_NOARGS,
      copy_doc},
-    {"count",        (PyCFunction)bitarray_count,       METH_O,
+    {"count",        (PyCFunction) bitarray_count,       METH_O,
      count_doc},
-    {"__deepcopy__", (PyCFunction)bitarray_copy,        METH_O,
+    {"__deepcopy__", (PyCFunction) bitarray_copy,        METH_O,
      copy_doc},
-    {"__delitem__",  (PyCFunction)bitarray_delitem,     METH_O,
+    {"__delitem__",  (PyCFunction) bitarray_delitem,     METH_O,
      0},
-    {"decode",       (PyCFunction)bitarray_decode,      METH_O,
+    {"decode",       (PyCFunction) bitarray_decode,      METH_O,
      decode_doc},
-    {"encode",       (PyCFunction)bitarray_encode,      METH_VARARGS,
+    {"encode",       (PyCFunction) bitarray_encode,      METH_VARARGS,
      encode_doc},
-    {"endian",       (PyCFunction)bitarray_endian,      METH_NOARGS,
+    {"endian",       (PyCFunction) bitarray_endian,      METH_NOARGS,
      endian_doc},
-    {"extend",       (PyCFunction)bitarray_extend,      METH_O,
+    {"extend",       (PyCFunction) bitarray_extend,      METH_O,
      extend_doc},
-    {"fill",         (PyCFunction)bitarray_fill,        METH_NOARGS,
+    {"fill",         (PyCFunction) bitarray_fill,        METH_NOARGS,
      fill_doc},
-    {"fromfile",     (PyCFunction)bitarray_fromfile,    METH_VARARGS,
+    {"fromfile",     (PyCFunction) bitarray_fromfile,    METH_VARARGS,
      fromfile_doc},
-    {"fromstring",   (PyCFunction)bitarray_fromstring,  METH_O,
+    {"fromstring",   (PyCFunction) bitarray_fromstring,  METH_O,
      fromstring_doc},
-    {"__getitem__",  (PyCFunction)bitarray_getitem,     METH_O,
+    {"__getitem__",  (PyCFunction) bitarray_getitem,     METH_O,
      0},
-    {"index",        (PyCFunction)bitarray_index,       METH_O,
+    {"index",        (PyCFunction) bitarray_index,       METH_O,
      index_doc},
-    {"insert",       (PyCFunction)bitarray_insert,      METH_VARARGS,
+    {"insert",       (PyCFunction) bitarray_insert,      METH_VARARGS,
      insert_doc},
-    {"invert",       (PyCFunction)bitarray_invert,      METH_NOARGS,
+    {"invert",       (PyCFunction) bitarray_invert,      METH_NOARGS,
      invert_doc},
-    {"__len__",      (PyCFunction)bitarray_length,      METH_NOARGS,
+    {"__len__",      (PyCFunction) bitarray_length,      METH_NOARGS,
      len_doc},
-    {"length",       (PyCFunction)bitarray_length,      METH_NOARGS,
+    {"length",       (PyCFunction) bitarray_length,      METH_NOARGS,
      length_doc},
-    {"pack",         (PyCFunction)bitarray_pack,        METH_O,
+    {"_multiply",    (PyCFunction) bitarray_multiply,    METH_O,
+     multiply_doc},
+    {"pack",         (PyCFunction) bitarray_pack,        METH_O,
      pack_doc},
-    {"pop",          (PyCFunction)bitarray_pop,         METH_VARARGS,
+    {"pop",          (PyCFunction) bitarray_pop,         METH_VARARGS,
      pop_doc},
-    {"__reduce__",   (PyCFunction)bitarray_reduce,      METH_NOARGS,
+    {"__reduce__",   (PyCFunction) bitarray_reduce,      METH_NOARGS,
      reduce_doc},
-    {"remove",       (PyCFunction)bitarray_remove,      METH_O,
+    {"remove",       (PyCFunction) bitarray_remove,      METH_O,
      remove_doc},
-    {"reverse",      (PyCFunction)bitarray_reverse,     METH_NOARGS,
+    {"reverse",      (PyCFunction) bitarray_reverse,     METH_NOARGS,
      reverse_doc},
-    {"setall",       (PyCFunction)bitarray_setall,      METH_O,
+    {"setall",       (PyCFunction) bitarray_setall,      METH_O,
      setall_doc},
-    {"__setitem__",  (PyCFunction)bitarray_setitem,     METH_VARARGS,
+    {"__setitem__",  (PyCFunction) bitarray_setitem,     METH_VARARGS,
      0},
-    {"sort",         (PyCFunction)bitarray_sort,        METH_NOARGS,
+    {"sort",         (PyCFunction) bitarray_sort,        METH_VARARGS |
+                                                         METH_KEYWORDS,
      sort_doc},
-    {"tofile",       (PyCFunction)bitarray_tofile,      METH_O,
+    {"tofile",       (PyCFunction) bitarray_tofile,      METH_O,
      tofile_doc},
-    {"tolist",       (PyCFunction)bitarray_tolist,      METH_NOARGS,
+    {"tolist",       (PyCFunction) bitarray_tolist,      METH_NOARGS,
      tolist_doc},
-    {"tostring",     (PyCFunction)bitarray_tostring,    METH_NOARGS,
+    {"tostring",     (PyCFunction) bitarray_tostring,    METH_NOARGS,
      tostring_doc},
-    {"to01",         (PyCFunction)bitarray_to01,        METH_NOARGS,
+    {"to01",         (PyCFunction) bitarray_to01,        METH_NOARGS,
      to01_doc},
-    {"unpack",       (PyCFunction)bitarray_unpack,      METH_VARARGS |
-                                                        METH_KEYWORDS,
+    {"unpack",       (PyCFunction) bitarray_unpack,      METH_VARARGS |
+                                                         METH_KEYWORDS,
      unpack_doc},
     {NULL, NULL, 0, NULL}  /* sentinel */
 };
@@ -2133,7 +2199,7 @@ static PyTypeObject Bitarraytype = {
 typedef struct {
     PyObject_HEAD
     idx_t index;
-    bitarrayobject* bao;
+    bitarrayobject *bao;
 } bitarrayiterobject;
 
 static PyTypeObject BitarrayIter_Type;
