@@ -1098,10 +1098,17 @@ class MethodTests(unittest.TestCase, Util):
         a.sort()
         self.assertEqual(a, bitarray('0000111'))
         
+        a = bitarray('1101000')
+        a.sort(reverse=True)
+        self.assertEqual(a, bitarray('1110000'))
+
+        self.assertRaises(TypeError, a.sort, 'A')
+        
         for a in self.randombitarrays():
             ida = id(a)
-            a.sort()
-            self.assertEqual(a, bitarray(sorted(a.tolist())))
+            rev = randint(0, 1)
+            a.sort(rev)
+            self.assertEqual(a, bitarray(sorted(a.tolist(), reverse=rev)))
             self.assertEqual(id(a), ida)
 
 
